"""A test for normal mode"""


import logging
import random
import time

import pathlib2 as pathlib

from kisspack.common import common_task_params


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_params(global_params):
    """
    The parameters for executing the task

    """

    params = common_task_params()
    # required keys
    params['name'] = pathlib.Path(__file__).stem
    params['description'] = __doc__
    params['est_task_time'] = (global_params.get('iterations', 10) * 3) + 3
    params['priority'] = 7
    params['req_param_keys'].append("iterations")

    return params


def task_setup(global_params):
    """Setup action for this task."""

    logger.info("execute setup normal a")

    time.sleep(2)

    return


def task_main(global_params):
    """The main task"""

    logger.info("execute task_main normal a")
    result = "Passed"
    task_message = "Nothing to see here"

    avg_delay = 0

    for i in range(0, global_params['iterations']):
        logger.debug("normal_a loop %s", i)
        delay_time = random.random()*2 + 1
        avg_delay = (avg_delay + delay_time)/2

        time.sleep(delay_time)

        metadata = {"task_message": task_message,
                    "avg_delay_time": avg_delay,
                    "widget_a_sub": global_params.get('widget_a_sub', "not found"),
                    "new_param": global_params.get("a_new_param", "not found")}

    return {'result': result, 'metadata': metadata}


def test_teardown(global_params):
    """Teardown action for this task."""

    logger.info("execute teardown normal a")

    time.sleep(1)



"""
Common helper funcs

"""

from distutils.sysconfig import get_python_lib
import logging

import pathlib2 as pathlib
from kissats.schemas.schemas import load_schema

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

PARAM_LOCATION = pathlib.PurePath(get_python_lib(),
                                  "kisspack",
                                  "PARAMS")


def common_task_params():
    """
    Basic params for this package

    """

    yaml_loc = pathlib.PurePath(PARAM_LOCATION, "common_task_params.yaml")
    param_out = load_schema(yaml_loc)

    return param_out



"""Custom Exceptions"""

import logging


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class KissPackError(Exception):
    """Base exception for package"""
    pass



"""setup a new param in the global params"""


import logging
import random
import time

import pathlib2 as pathlib

from kisspack.common import common_task_params


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_params(global_params):
    """
    The parameters for executing the task

    """

    params = common_task_params()
    # required keys
    params['name'] = pathlib.Path(__file__).stem
    params['description'] = __doc__
    params['est_task_time'] = 6
    params['priority'] = 7

    return params


def task_main(global_params):
    """The main task"""

    logger.info("execute task_main setup set_param")
    result = "Passed"
    task_message = "Nothing to see here"

    delay_time = random.random()*5 + 1

    time.sleep(delay_time)

    global_params["a_new_param"] = "set by setup_set_param"

    metadata = {"task_message": task_message}

    return {'result': result, 'metadata': metadata}



"""
Setup sequences

"""

import logging


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_global_setup(params):
    """Global setup"""

    setup_list = ["setup_set_param"]

    return setup_list


def get_all_setup(params):
    """a list of all setup tasks"""

    setup_list = ['test_check_setup']

    setup_list.extend(get_normal_setup(params))
    setup_list.extend(get_thread_setup(params))
    setup_list.extend(get_proc_setup(params))

    return setup_list


def get_normal_setup(params):
    """normal mode setup"""

    setup_list = []

    return setup_list


def get_thread_setup(params):
    """thread mode setup"""

    setup_list = []

    return setup_list


def get_proc_setup(params):
    """proc mode setup"""

    setup_list = ["setup_proc"]

    return setup_list



"""Test group sequences"""


import logging


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_all_tests(params):
    """a list of all tests"""

    test_list = ['test_check_setup']

    test_list.extend(get_normal_tests(params))
    test_list.extend(get_thread_tests(params))
    test_list.extend(get_proc_tests(params))

    return test_list


def get_normal_tests(params):
    """normal mode tests"""

    test_list = ["test_normal_a",
                 "test_normal_b",
                 "test_normal_c",
                 "test_normal_d"]

    return test_list


def get_thread_tests(params):
    """thread mode tests"""

    test_list = ["test_thread_a",
                 "test_thread_b",
                 "test_thread_c",
                 "test_thread_d"]

    return test_list


def get_proc_tests(params):
    """proc mode tests"""

    test_list = ["test_proc_a",
                 "test_proc_b",
                 "test_proc_c",
                 "test_proc_d"]

    return test_list



"""A teardown test"""


import logging
import time

import pathlib2 as pathlib

from kisspack.common import common_task_params


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_params(global_params):
    """
    The parameters for executing the task

    """

    params = common_task_params()
    # required keys
    params['name'] = pathlib.Path(__file__).stem
    params['description'] = __doc__
    params['est_task_time'] = 2
    params['priority'] = 1

    return params


def task_main(global_params):
    """The main task"""

    logger.info("execute task_main teardown_clear_param")
    result = "Passed"
    task_message = "Nothing to see here"

    time.sleep(2)

    global_params.pop("a_new_param")

    metadata = {"task_message": task_message,
                "widget_a_sub": global_params.get('widget_a_sub', "not found")}

    return {'result': result, 'metadata': metadata}



"""
Teardown sequences

"""

import logging


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_global_teardown(params):
    """Global teardown"""

    teardown_list = ['teardown_global']

    return teardown_list


def get_thread_teardown(params):
    """teardown steps for thread"""

    teardown_list = ["teardown_clear_param"]

    return teardown_list



"""Setup for KISS ATS"""

from setuptools import (setup,
                        find_packages)

setup(
    name="kisspack",
    version="1.0.0a1",
    author="Bob Folkes",
    author_email="bob@nwcompnet.com",
    description=("A test package for KissATS"),
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Manufacturing",
        "Intended Audience :: Science/Research",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='ats simple test automation',
    packages=find_packages(exclude=['docs', 'tests*', '.git', '.vs']),
    package_data={"kisspack": ["params/*.yaml"]},
    install_requires=["kissats",
                      "pathlib2"]
)



"""A test to test prereq order"""


import logging
import random
import time

import pathlib2 as pathlib

from kisspack.common import common_task_params


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def get_params(global_params):
    """
    The parameters for executing the task

    """

    params = common_task_params()
    # required keys
    params['name'] = pathlib.Path(__file__).stem
    params['description'] = __doc__
    params['est_task_time'] = 8
    params['priority'] = 1
    params['prereq_tasks'] = list()

    return params


def task_setup(global_params):
    """Setup action for this task."""

    logger.info("execute setup check setup")

    time.sleep(2)

    return


def task_main(global_params):
    """The main task"""

    logger.info("execute task_main setup check")
    result = "Passed"
    task_message = "Nothing to see here"

    delay_time = random.random()*2 + 1
    time.sleep(delay_time)

    metadata = {"task_message": task_message,
                "widget_a_sub": global_params.get('widget_a_sub', "not found"),
                "new_param": global_params.get("a_new_param", "not found")}

    return {'result': result, 'metadata': metadata}


def test_teardown(global_params):
    """Teardown action for this task."""

    logger.info("execute teardown setup check")

    time.sleep(3)
